import 'dart:convert';
import 'dart:math';

class GameDataItem {
  final int? value;

  const GameDataItem({
    required this.value,
  });

  factory GameDataItem.fromValue(int? value) {
    return GameDataItem(
      value: value,
    );
  }

  factory GameDataItem.fromJson(Map<String, dynamic>? json) {
    return GameDataItem(
      value: (json?['value'] != null) ? (json?['value'] as int) : null,
    );
  }

  Map<String, dynamic>? toJson() {
    return <String, dynamic>{
      'value': this.value,
    };
  }

  String toString() {
    return jsonEncode(this.toJson());
  }
}

class GameData {
  final bool isReady;
  final int boardSize;
  final List<List<GameDataItem>> board;

  const GameData({
    required this.isReady,
    required this.boardSize,
    required this.board,
  });

  factory GameData.createNew(int boardSize) {
    final List<List<GameDataItem>> cells = [];

    for (var y = 0; y < boardSize; y++) {
      final List<GameDataItem> line = [];
      for (var x = 0; x < boardSize; x++) {
        final GameDataItem item = new GameDataItem(value: null);
        line.add(item);
      }

      cells.add(line);
    }

    return GameData(
      isReady: true,
      boardSize: boardSize,
      board: cells,
    );
  }

  factory GameData.createRandom(int boardSize) {
    const allowedValues = [0, 1, 2, 3, 4, 5];
    final allowedValuesSize = allowedValues.length;

    final List<List<GameDataItem>> cells = [];

    for (var y = 0; y < boardSize; y++) {
      final List<GameDataItem> line = [];
      for (var x = 0; x < boardSize; x++) {
        final value = allowedValues[Random().nextInt(allowedValuesSize)];
        final GameDataItem item = new GameDataItem(value: value);
        line.add(item);
      }

      cells.add(line);
    }

    return GameData(
      isReady: true,
      boardSize: boardSize,
      board: cells,
    );
  }

  GameDataItem getCell(int x, int y) {
    return this.board[y][x];
  }

  int? getCellValue(int x, int y) {
    return this.getCell(x, y).value;
  }

  void updateCellValue(int x, int y, int? value) {
    this.board[y][x] = new GameDataItem.fromValue(value);
  }

  factory GameData.fromJson(Map<String, dynamic>? json) {
    return GameData(
      isReady: (json?['isReady'] != null) ? (json?['isReady'] as bool) : false,
      boardSize: (json?['boardSize'] != null) ? (json?['boardSize'] as int) : 0,
      board: (json?['board'] != null) ? (json?['board'] as List<List<GameDataItem>>) : [],
    );
  }

  Map<String, dynamic>? toJson() {
    return <String, dynamic>{
      'isReady': this.isReady,
      'boardSize': this.boardSize,
      'board': this.board,
    };
  }

  String toString() {
    return jsonEncode(this.toJson());
  }
}
